'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapCalc = exports.stretchRatio = exports.stretchCount = exports.stretchValue = exports.isStretch = undefined;

exports.default = function () {
  var _this = this;

  this.iterateDirections(function (direction, lengths) {
    if (lengths.some(_isAuto2.default) || _this.isAlignedInDirection(direction)) {
      return;
    }

    var stretchValues = lengths.filter(isStretch).map(stretchValue);
    var totalStretch = addAll(stretchValues);

    var remainingValues = lengths.filter(function (length) {
      return !isStretch(length);
    });
    var totalRemaining = addAll(remainingValues) || '0';

    lengths.forEach(function (length, i) {
      if (isStretch(length)) {
        // (99.9% - totalRemaining) * thisStretch / totalStretch
        var expression = '(99.9% - ' + totalRemaining + ') * ' + stretchValue(length) + ' / ' + totalStretch;

        lengths[i] = wrapCalc(expression);
      }
    });

    _this.setDirection(direction, lengths);
  });

  return this;
};

var _isAuto = require('./isAuto');

var _isAuto2 = _interopRequireDefault(_isAuto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Tell if `length` is a stretch length
var isStretch = exports.isStretch = function isStretch(length) {
  var unit = length.match(/[A-z]+/);
  return unit !== null && unit[0] === 's';
};

// Get the number before the 's' unit
var stretchValue = exports.stretchValue = function stretchValue(length) {
  if (!isStretch(length)) {
    throw new Error(length + ' is not a stretch length');
  }
  return length.slice(0, -1);
};

// How much of these lengths are stretch ?
var stretchCount = exports.stretchCount = function stretchCount(lengths) {
  return lengths.filter(isStretch).length;
};

// Get the ratio between the first and last stretch length
var stretchRatio = exports.stretchRatio = function stretchRatio(lengths) {
  var first = stretchValue(lengths[0]);
  var last = stretchValue(lengths[2]);
  return first + ' * 99.9% / (' + first + ' + ' + last + ')';
};

// Wrap calc( ) around a string
var wrapCalc = exports.wrapCalc = function wrapCalc(length) {
  return 'calc(' + length + ')';
};

var addAll = function addAll(array) {
  if (array.length === 0) {
    return undefined;
  } else {
    var str = '(' + array[0];
    return array.slice(1).reduce(function (a, b) {
      return a + ' + ' + b;
    }, str) + ')';
  }
};

// Turn all stretch lengths into valid calc expressions