'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _position = require('./position/');

var _position2 = _interopRequireDefault(_position);

var _isAuto = require('./position/isAuto');

var _isAuto2 = _interopRequireDefault(_isAuto);

var _stretch = require('./position/stretch');

var _transform = require('./transform');

var _transform2 = _interopRequireDefault(_transform);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Processes positioning for a rule

exports.default = function (rule) {
  var position = _position2.default.parse(rule).stretch();

  var prefix = '';

  if (position.align) {
    // display: block or inline-block depending on the axis
    rule.append({
      prop: 'display',
      value: position.align.display
    });

    prefix = 'margin-';

    // first and last margins
    if (position.margins) {
      rule.after({
        selector: rule.selector + ':first-child'
      }).next().append({
        prop: 'margin-' + position.align.before,
        value: position.margins.first
      }).after({
        selector: rule.selector + ':last-child'
      }).next().append({
        prop: 'margin-' + position.align.after,
        value: position.margins.last
      });
    }
  } else {
    // Declarations with auto and 2 stretch lengths
    (0, _transform.resetDeclaration)();
    position.iterateDirections(function (direction, lengths) {
      if (lengths.some(_isAuto2.default) && (0, _stretch.stretchCount)(lengths) === 2) {
        position.type = 'relative';

        var ratio = (0, _stretch.stretchRatio)(lengths);
        rule.append({
          prop: direction.before,
          value: (0, _stretch.wrapCalc)(ratio)
        });
        (0, _transform2.default)(rule, direction, ratio);

        position.setDirection(direction, [undefined, lengths[1], undefined]);
      }
    });

    position.strip();

    // position: absolute / relative / static / fixed
    rule.append({
      prop: 'position',
      value: position.type || 'absolute'
    });
  }

  position.iterateLengths(function (direction, length, value) {
    rule.append({
      prop: (length === 'size' ? '' : prefix) + direction[length],
      value
    });
  });
};